// Module Name: maze.cpp
// Author:      Stephen Manley (smanley@nyx.net)
//
// Description:
//
// This class is the maze object. It is comprised of two
// planes representing a floor and ceiling, a set of 
// X and Y dimensions, a rotation about it's axis, and
// a display scaling factor.
//
///////////////////////////////////////////////////////////////////////////

#ifndef MAZE
#include "maze.h"
#endif

//#include <windows.h>
#include <GL/gl.h>
#include <GL/glut.h>
#include <iostream.h>
#include <fstream.h>


Maze::Maze()
{
	// Rotation, scale and translation information
	xRotation = 45.0;
	yRotation = 0.0;

	xScale = 1.2;
	yScale = 1.2;
	zScale = 1.2;

	xSize = 20;
	ySize = 20;

	// Wait until the tiles have been processed!
	mazeEnabled = 0;
}

Maze::~Maze()
{
}

// Loads the maze from disk
void Maze::load()
{
    fstream inFile("grid.dat", ios::in);
    
	if (!inFile)
		return;

    // Read in the grid dimensions
    inFile >> xSize;
    inFile >> ySize;
    colorArray = new double[xSize*ySize*3];

    double r,g,b;

	// Read in the colors of each tile (X*Y)
    for (int i=0; i<xSize*ySize*3; i++)
    {
        inFile >> r;
        colorArray[i] = r;
        
        inFile >> g;
		colorArray[(++i)] = g;

        inFile >> b;
		colorArray[(++i)] = b;
    }

    inFile.close();

}

// Sets the animation function
void Maze::animate(int degrees)
{
	yRotation += degrees;
}

// Draws the maze
void Maze::draw()
{
   // Clear the old display
   glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   // Apply model transformations
   glPushMatrix();

   glTranslated(0,0,-60);
   glRotated(xRotation, 1., 0., 0.);
   glRotated(yRotation, 0., 0., 1.);
   glScaled(xScale, yScale, zScale);

   Wall* myWall = new Wall(); 

   // Draw the ceiling (0.01 to win roundoff with wall)
   drawPlane(0.01);

   int i = -1;
   int tilesize = 1;
   double z = 0; // Ceiling Plane
   double z2 = -5; // Floor Plane


   if (mazeEnabled)
   {
	   for (int j=0; j<1600; j++)
			mazeWalls[j]->draw();
   }


   // Draw the walls
   for (int x=-10; x<xSize-10; x++)
   {
       for (int y=-10; y<ySize-10; y++)
       {
          double r = colorArray[(++i)]; 
          double b = colorArray[(++i)];
          double g = colorArray[(++i)];

		  // Depending on what color was selected in the 2D maze creator,
		  // draw it with an appropriately styled tile.
          if (r == 1)
		  {              
			  // Top and bottom
   		      myWall->drawWall3(x,y,z, x,y,z2, x+1,y,z2, x+1,y,z );
   		      myWall->drawWall3(x,y-1,z, x,y-1,z2, x+1,y-1,z2, x+1,y-1,z );

              // Sides
   		      myWall->drawWall3(x+1,y,z, x+1,y,z2, x+1,y-1,z2, x+1,y-1,z);
   		      myWall->drawWall3(x,y,z, x,y,z2, x,y-1,z2, x,y-1,z);
   		  }
   		  
          if (g == 1)
		  {
			  // Top and bottom
   		      myWall->drawWall2(x,y,z, x,y,z2, x+1,y,z2, x+1,y,z );
   		      myWall->drawWall2(x,y-1,z, x,y-1,z2, x+1,y-1,z2, x+1,y-1,z );

              // Sides
   		      myWall->drawWall2(x+1,y,z, x+1,y,z2, x+1,y-1,z2, x+1,y-1,z);
   		      myWall->drawWall2(x,y,z, x,y,z2, x,y-1,z2, x,y-1,z);
   		  }
          if (b == 1)
		  {
			  // Top and bottom
   		      myWall->drawWall1(x,y,z, x,y,z2, x+1,y,z2, x+1,y,z );
   		      myWall->drawWall1(x,y-1,z, x,y-1,z2, x+1,y-1,z2, x+1,y-1,z );

              // Sides
   		      myWall->drawWall1(x+1,y,z, x+1,y,z2, x+1,y-1,z2, x+1,y-1,z);
   		      myWall->drawWall1(x,y,z, x,y,z2, x,y-1,z2, x,y-1,z);
   		  }

       }
   }

   // Draw the ceiling
   drawPlane(-5.01);

   glPopMatrix();

   delete myWall;
}

// Draw a "Ceiling" or "Floor"
void Maze::drawPlane(double z)
{
   int tilesize = 1;
   int xSize = 20;
   int ySize = 20;
   
   //double z = -20;

   glColor3f(1.0, 0.0, 0.0);

   int i = -1;

   glBegin(GL_QUADS);
   
   for (int x=-10; x<xSize-10; x++)
   {
       for (int y=-10; y<ySize-10; y++)
       {
          double r = colorArray[(++i)]; 
          double b = colorArray[(++i)];
          double g = colorArray[(++i)];

          if ((r == 1) || (g == 1) || (b == 1))
		  {
		  	  glColor3f(.5, .5, .5);	
          
   		      glVertex3d(x,y,z);
   		      glVertex3d(x+tilesize, y, z);
   		      glVertex3d(x+tilesize, y-tilesize, z);
   		      glVertex3d(x, y-tilesize, z); 

   		  }
       }
   }

   glEnd();
}

// Generate the maze tiles
void Maze::generateMaze()
{
	// Load the maze from disk
	load();

/*  THIS IS MY BROKEN HIDDEN WALL REMOVAL CODE, SEE NOTE

	// Allocate space for the maximum number of tiles 
	// This could be optimized!
	for (int i=0; i<1600; i++)
	{
		mazeWalls[i] = new Wall(0,0,0, 0,0,0, 0,0,0, 0,0,0);
		mazeWalls[i]->setActive(0);
	}

	// Enable tile drawing
	mazeEnabled = 1;

	// Determine how many tiles we need and set states
	double r, g, b;
	double rCheck, gCheck, bCheck;


	int needsTop = 1;
	int needsBottom = 1;
	int needsLeft = 1;
	int needsRight = 1;

	int xPos, yPos;
	int tileCount = 0;

	double z = 0; // Ceiling Plane
	double z2 = -5; // Floor Plane


	for (int y=1; y<ySize; y++) // ignore outer parameter!
	{
	   for (int x=1; x<xSize; x++)
	   {
			xPos = x + 2;
			yPos = y;

		    r = colorArray[((y+10) * ySize) + ((x)+10)];
			b = colorArray[((y+10) * ySize) + ((x++)+10)];
			g = colorArray[((y+10) * ySize) + ((x++)+10)];


			// Check the 4 ajacent corners
			if ((r == 1) || (b == 1) || (g == 1))
			{
/*
				// Top
				rCheck = colorArray[((yPos-1) * ySize) + ((xPos))];
				bCheck = colorArray[((yPos-1) * ySize) + ((xPos+1))];
				gCheck = colorArray[((yPos-1) * ySize) + ((xPos+2))];
				
				if ((rCheck == 1) || (bCheck == 1) || (gCheck == 1))
					needsTop = 0;

				// Bottom
				rCheck = colorArray[((yPos+1) * ySize) + ((xPos))];
				bCheck = colorArray[((yPos+1) * ySize) + ((xPos))];
				gCheck = colorArray[((yPos+1) * ySize) + ((xPos))];

				if ((rCheck == 1) || (bCheck == 1) || (gCheck == 1))
					needsBottom = 0;

				// Left
				rCheck = colorArray[((yPos) * ySize) + ((xPos+3))];
				bCheck = colorArray[((yPos) * ySize) + ((xPos+4))];
				gCheck = colorArray[((yPos) * ySize) + ((xPos+5))];

				if ((rCheck == 1) || (bCheck == 1) || (gCheck == 1))
					needsLeft = 0;

				// Right
				rCheck = colorArray[((yPos) * ySize) + ((xPos-5))];
				bCheck = colorArray[((yPos) * ySize) + ((xPos-4))];
				gCheck = colorArray[((yPos) * ySize) + ((xPos-3))];
				
				if ((rCheck == 1) || (bCheck == 1) || (gCheck == 1))
					needsRight = 0;

				// for (int x=-10; x<xSize-10; x++)
				// for (int y=-10; y<ySize-10; y++)
     
 			    
				double x1 = ((int)xPos/3) - 10 -1;
				double y1 = yPos - 10 -1;
				

				// Top and bottom
				tileCount++;
 				mazeWalls[tileCount] = new Wall(x1,y1,z, x1,y1,z2, x1+1,y1,z2, x1+1,y1,z );
   				mazeWalls[tileCount]->setActive(1);
				mazeWalls[tileCount]->setType(2);

				tileCount++;
				mazeWalls[tileCount] = new Wall(x1,y1-1,z, x1,y1-1,z2, x1+1,y1-1,z2, x1+1,y1-1,z );
   				mazeWalls[tileCount]->setActive(1);
				mazeWalls[tileCount]->setType(2);

				// Sides
				tileCount++;
   				mazeWalls[tileCount] = new Wall(x1+1,y1,z, 1+1,y1,z2, x1+1,y1-1,z2, x1+1,y1-1,z);
   				mazeWalls[tileCount]->setActive(1);
				mazeWalls[tileCount]->setType(2);
   				
				tileCount++;
				mazeWalls[tileCount] = new Wall(x1,y1,z, x1,y1,z2, x1,y1-1,z2, x1,y1-1,z);
    			mazeWalls[tileCount]->setActive(1);
				mazeWalls[tileCount]->setType(2);
	
			 }   

		}
	}
*/
}

void Maze::addScale(double x, double y, double z)
{
	xScale += x;
	yScale += y;
	zScale += z;
}

void Maze::addRotation(double x, double y, double z)
{
	xRotation += x;
	//yRotation += y;
	//xRotation += z;
}

void Maze::setRotation(double x, double y, double z)
{
	xRotation += x;
	yRotation += y;
	xRotation += z;
}


void Maze::setScale(double x, double y, double z)
{
	xScale = x;
	yScale = y;
	zScale = z;
}

void Maze::getRotation(double &x, double &y, double &z)
{
	x = xRotation;
	y = yRotation;
	z = zRotation;
}

void Maze::getScale(double &x, double &y, double &z)
{
	x = xScale;
	y = yScale;
	z = zScale;
}

