// Module Name: maze.cpp
// Author:      Stephen Manley (smanley@nyx.net)
//
// Description:
// 
// This program generates a maze from the data files created with the 
// mazemaker. It takes the 2D maze information which is stored in 
// terms of an array of color values and generates a 3D maze from it.
// This 3D maze is comprised of walls, drawn with one of three methods.
// There is basic C++ class abstraction, but since it wasn't emphasised
// on the assignment, I didn't go crazy.
//
// The program loads the maze, computes what walls need to go where, 
// then stores the information in the maze for optimal redraw (no 
// additional computation.
// 
// NOTE: I got a bug in my code and I wasn't able to fix it in time, so
// my hidden wall removal code doesn't work properly. This wasn't
// specified in the assignment as a requirement, so I don't think it's
// that big a deal. Obviously, this will have to be fixed for assignment
// #3. The maze appears fine when being rotated.
// 
// Usage:
// 
// Pressing l will load the maze into the maze generator and automatically
// display the new 3D generated maze. The file must be stored in:
// "./grid.dat" This is also done automagically on startup.
// 
// To maniuplate the maze, q, w, a, s rotate in the X and Y parameters. 
// c and v scales the maze. Alternately, pressing and holding the mouse
// button and moving up/dowwn will tilt the maze. The maze automatically
// rotates about the axis. Moving the mouse in the horizontal direction
// will change the scale of the drawing.
// 
// My machine is quite slow, so the keyboard controls work much better
// than the mouse, although the mouse controls work fine (Just not
// realtime on my machine). 
// 
// NOTE ON MOUSE CONTROL: For optimal use, click the left button in the
// middle of the window. Then slowly move towards the edge of the screen.
// The response isn't exactly linear, but this approach made for the 
// most managable combination of scale and rotate on my slow machine.
//
////////////////////////////////////////////////////////////////////////////

// Eeeewwwww
//#include <windows.h>

// System includes
#include <GL/gl.h>
#include <GL/glut.h>
#include <iostream.h>
#include <fstream.h>

// Object Includes
#ifndef MAZE
#include "maze.h"
#endif

#ifndef WALL
#include "wall.h"
#endif

// Height and width of the active window
int windowHeight, windowWidth;

// Mouse status variables
int mouseClickX, mouseClickY;
int leftMouseDown = 0;

// Object pointers
Maze* maze;
Wall* wall;

// Configure projection information
void init(void) 
{
   glClearColor (0.0, 0.0, 0.0, 0.0);
   glShadeModel (GL_SMOOTH);

   // Initialize projection
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective(45.0, 1.0, 1.0, 100.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   // Make sure we clip hidden surfaces!
   glEnable(GL_DEPTH_TEST);
}

// Redraw the whole scene
void display(void)
{
	maze->draw();

	// Swap and flush the buffers
	glutSwapBuffers();

}

// Reshape the window - save the parameters
void reshape (int w, int h)
{
   glViewport (0, 0, (GLsizei) w, (GLsizei) h);

   windowWidth = w;
   windowHeight = h; 
}

// Loads the maze from the files created by mazemaker into a 2D array
void loadMaze()
{
	maze->generateMaze();
}

// Keyboard handler
void keyboard(unsigned char key, int x, int y)
{
	double xR, yR, zR;
	double xS, yS, zS;
	maze->getRotation(xR, yR, zR);
	maze->getScale(xS, yS, zS);


   switch (key) {
      case 27:
         exit(0);
         break;

      case 'l':
         loadMaze();
         display(); // redraw
         break;   

      case 'c':
         xS += .1;
         yS += .1;
         zS += .1;
		 maze->setScale(xS, yS, zS);
         
         display();
         break;

      case 'v':
         xS -= .1;
         yS -= .1;
         zS -= .1;
		 maze->setScale(xS, yS, zS);
         
         display();
         break;

      case 'q':
		 xR += 5;
		 maze->setRotation(xR, yR, zR);

		 display();
      	 break;

      case 'w': 	 
		 xR -= 5;
 		 maze->setRotation(xR, yR, zR);
   	 
		 display();
      	 break; 

      case 'a':
         yR += 5;
		 maze->setRotation(xR, yR, zR);

         display();
         break;	 

      case 's':
         yR -= 5;
		 maze->addRotation(xR, yR, zR);

         display();
         break;	 
   }
}

// Mouse motion callback
void mouseMove(int x, int y)
{
	if (leftMouseDown)
	{
		// Calculate a delta X and delta Y
		double dx = mouseClickX - x;
		double dy = mouseClickY - y;

		// Allow a maximum scale and rotation
		double dScale =  abs((dx/((double)windowWidth) * 3.0));
		double dRotate = abs((dy/((double)windowWidth) * 20.0));

		double xR, yR, zR;
		double xS, yS, zS;

		//maze->getRotation(xR, yR, zR);
		//maze->getScale(xS, yS, zS);

		// Left right changes scale - put some careful bounds here
		if(mouseClickX < x) 
		{
			maze->addScale(dScale, dScale, dScale);
		}

		if(mouseClickX > x)
		{
			maze->addScale(-dScale, -dScale, -dScale);
		}

		// Up down changes the tilt on the times
		if (mouseClickY < y)
		{
			maze->addRotation(dRotate, 0, 0);
		}

		if (mouseClickY > y)
		{
			maze->addRotation(-dRotate, 0, 0);
		}

		//maze->setRotation(xR, yR, zR);
		//maze->setScale(xS, yS, zS);
		
		// Update the display in realtime
		display();
	}
}

// Rotates the display in a timed animation
void animate(int degrees)
{    
	maze->animate(degrees);

	// Don't go too fast so events can happen
	display();
    glutTimerFunc(50, animate, 5);
}

// Snags mouse clicks for the mouse move callback
void mouse(int button, int state, int mouseX, int mouseY)
{
	// Snag the origin point for the control drag
    if (button == GLUT_LEFT_BUTTON)
	{
   		if (state == GLUT_DOWN)
   		{
			if (!leftMouseDown)
			{
				mouseClickX = mouseX;
	   			mouseClickY = mouseY;
			}
   		
   		    leftMouseDown = 1;
   		}
		
		if (state == GLUT_UP)
		{
			leftMouseDown = 0;
		}
   }
}

int main(int argc, char** argv)
{
	// Create scene objects
    maze = new Maze();
	wall = new Wall();

    loadMaze();

	// Glut init
	glutInit(&argc, argv);
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize (500, 500); 
	glutInitWindowPosition (100, 100);
	glutCreateWindow ("MazeGen 3D");

	init ();

	// Callbacks
	glutDisplayFunc(display); 
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutTimerFunc(50, animate, 5);   
	glutMouseFunc(mouse);
	glutMotionFunc(mouseMove);
	glutMainLoop();
   
	return 0;
}




